#include "StdAfx.h"
#include "HookProcs.h"
#include "Macros.h"
#include <Tlhelp32.h>
#include <shlwapi.h>
#include <vector>

#pragma intrinsic(memcmp, memcpy, memset) 

void WriteProtectedMemory(void* dest_addr, void* src_addr, size_t size)
{
	BOOL br;
	SIZE_T vr;
	DWORD old_value;
	MEMORY_BASIC_INFORMATION mbi;

	vr=::VirtualQuery(dest_addr, &mbi, sizeof(MEMORY_BASIC_INFORMATION));
	return_if_not_equal(vr,sizeof(MEMORY_BASIC_INFORMATION));
	br=::VirtualProtect(mbi.BaseAddress,mbi.RegionSize,PAGE_WRITECOPY,&mbi.Protect);
	return_if_equal(br,FALSE);
	CopyMemory(dest_addr,src_addr,size);
	::VirtualProtect(mbi.BaseAddress,mbi.RegionSize,mbi.Protect,&old_value);
	return_if_equal(br,FALSE);
}


bool AddHook_ExportTable_ModuleBase(void* hook_target_module_base, char* proc_name, FARPROC new_proc, FARPROC* old_proc)
{
	BYTE* dll_addr=(BYTE*)hook_target_module_base;

	IMAGE_DOS_HEADER* dos_header=(IMAGE_DOS_HEADER*)dll_addr;
	return_x_if_not_equal(dos_header->e_magic,0x5A4D,false);// check MZ
	IMAGE_NT_HEADERS* pe_header=(IMAGE_NT_HEADERS*)(dll_addr+ dos_header->e_lfanew);
	return_x_if_not_equal(pe_header->Signature,0x4550,false);// check PE

	DWORD rva_exports = pe_header->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress;
	IMAGE_EXPORT_DIRECTORY*	va_exports = (IMAGE_EXPORT_DIRECTORY*)(dll_addr+ rva_exports);

	void* va_names=(dll_addr+ va_exports->AddressOfNames);
	void* va_functions = (dll_addr+ va_exports->AddressOfFunctions);
	void* va_name_ordinals = (dll_addr+ va_exports->AddressOfNameOrdinals);

	size_t* rva_name=(size_t*)va_names;
	WORD* rva_ordinal=(WORD*)va_name_ordinals;
	size_t* rva_function=(size_t*)va_functions;

	for( size_t i=0; i<va_exports->NumberOfFunctions; i++ )
	{
		char* name=(char*)(dll_addr + rva_name[i]);

		if( 0 == lstrcmp(name,proc_name) )
		{
			size_t* p_exported_rva=&rva_function[rva_ordinal[i]];
			size_t hook_va=(size_t)(new_proc);
			size_t hook_rva=hook_va-size_t(dll_addr);

			if(old_proc!=NULL)
				*old_proc=(FARPROC)(size_t(dll_addr) + (*p_exported_rva) );
			WriteProtectedMemory(p_exported_rva,&hook_rva,sizeof(hook_rva));
			return true;
		}
	}
	return false;
}

bool AddHook_ExportTable_ModuleName(LPCTSTR hook_target_module_name, char* proc_name, FARPROC new_proc, FARPROC* old_proc)
{
	HMODULE dll=GetModuleHandle(hook_target_module_name);
	return_x_if_equal(dll,NULL,false);
	BYTE* dll_addr=(BYTE*)((size_t)dll & 0xFFFF0000);

	return AddHook_ExportTable_ModuleBase(dll_addr,proc_name,new_proc,old_proc); 
}

void AddHook_ImportTable_ModuleBase(void* hook_target_module_base, LPCTSTR module_which_provide_proc, 
									char* exported_proc_name, FARPROC new_proc, FARPROC* old_proc)
{
	BYTE* dll_addr=(BYTE*)hook_target_module_base;

	if(old_proc!=NULL)
		*old_proc=NULL;

	IMAGE_DOS_HEADER* dos_header=(IMAGE_DOS_HEADER*)dll_addr;
	return_if_not_equal(dos_header->e_magic,0x5A4D);// check MZ
	IMAGE_NT_HEADERS* pe_header=(IMAGE_NT_HEADERS*)(dll_addr+ dos_header->e_lfanew);
	return_if_not_equal(pe_header->Signature,0x4550);// check PE

	DWORD rva_imports = pe_header->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT].VirtualAddress;
	if(rva_imports==0)
		return;// No imports its okay
	IMAGE_IMPORT_DESCRIPTOR* va_imports = (IMAGE_IMPORT_DESCRIPTOR*)(dll_addr+ rva_imports);

	IMAGE_IMPORT_DESCRIPTOR zero_imports;
	ZeroMemory(&zero_imports,sizeof(zero_imports));
	for( size_t di=0; 0 != memcmp(&va_imports[di],&zero_imports,sizeof(zero_imports)); di++ )
	{
		char* import_dll_name=(char*)((BYTE*) dll_addr + va_imports[di].Name);
		IMAGE_THUNK_DATA* va_thunk=(IMAGE_THUNK_DATA*)( (BYTE*) dll_addr + va_imports[di].FirstThunk);
		size_t* rva_import_proc_name=((size_t*)((BYTE*) dll_addr + va_imports[di].OriginalFirstThunk));

		for( size_t pi=0; true; pi++ )
		{
			IMAGE_IMPORT_BY_NAME* va_import_proc_name=(IMAGE_IMPORT_BY_NAME*)( (BYTE*) dll_addr + rva_import_proc_name[pi]);

			if(rva_import_proc_name[pi]==NULL)
				break;
			if(va_thunk[pi].u1.Function==NULL)
				break;

			char* proc_name=(char*)va_import_proc_name->Name;

			if( (0 == lstrcmpi(import_dll_name,module_which_provide_proc)) && 
				(0 == lstrcmp(proc_name,exported_proc_name)) )
			{
				size_t* p_exported_va=(size_t*)(&va_thunk[pi].u1.Function);
				size_t hook_va=(size_t)(new_proc);
				if(old_proc!=NULL)
					*old_proc=(FARPROC)(*p_exported_va);
				WriteProtectedMemory(p_exported_va,&hook_va,sizeof(hook_va));
				return;
			}
		}
	}
}

void AddHook_ImportTable_ModuleName(LPCTSTR hook_target_module_name, LPCTSTR module_which_provide_proc, 
									char* exported_proc_name, FARPROC new_proc, 
									FARPROC* old_proc)
{
	HMODULE dll=GetModuleHandle(hook_target_module_name);
	BYTE* dll_addr=(BYTE*)((size_t)dll & 0xFFFF0000);

	AddHook_ImportTable_ModuleBase( dll_addr, module_which_provide_proc, 
		exported_proc_name, new_proc, old_proc );
}

void RunRemoteLibrary(DWORD target_process_id, bool load_lybrary, void* remote_param)
{
	HANDLE	remote_process;
	HANDLE	remote_thread;
	HMODULE remote_kernel32;
	FARPROC remote_thread_entry;
	DWORD	remote_result=0;

	remote_kernel32=::GetModuleHandle(_T("Kernel32"));
	if(load_lybrary)
		remote_thread_entry = ::GetProcAddress(remote_kernel32, "LoadLibraryA");
	else
		remote_thread_entry = ::GetProcAddress(remote_kernel32, "FreeLibrary");
	return_if_equal(remote_thread_entry,NULL);
	remote_process=OpenProcess(PROCESS_ALL_ACCESS,FALSE,target_process_id);
	remote_thread = ::CreateRemoteThread(remote_process,NULL, 0, 
		(LPTHREAD_START_ROUTINE)remote_thread_entry, 
		remote_param, 0, NULL);
	CloseHandle(remote_process);
	return_if_equal(remote_thread,NULL);
	::WaitForSingleObject(remote_thread, INFINITE);
	GetExitCodeThread(remote_thread,&remote_result);
	CloseHandle(remote_thread);
	return_if_equal(remote_result,NULL);
}

void HookProcs::InjectModule(DWORD target_process_id, const char* module_file_absolute_path)
{
	BOOL	br;
	char*	file_name;
	size_t	file_name_size;
	char*	remote_file_name;
	HANDLE	remote_process;

	file_name=(char*)module_file_absolute_path;
	file_name_size = lstrlen(file_name)+1;

	remote_process=OpenProcess(PROCESS_VM_OPERATION|PROCESS_VM_WRITE,FALSE,target_process_id);
	remote_file_name = (char*)::VirtualAllocEx(remote_process, NULL, 
		file_name_size, MEM_COMMIT, PAGE_READWRITE);
	return_if_equal(remote_file_name,NULL);
	br=::WriteProcessMemory(remote_process, remote_file_name, file_name, file_name_size, NULL);
	return_if_equal(br,FALSE);

	RunRemoteLibrary(target_process_id,true,remote_file_name);

	br=::VirtualFreeEx(remote_process,remote_file_name,file_name_size,MEM_DECOMMIT);
	assert_if_equal(br,FALSE);
	CloseHandle(remote_process);
}

void HookProcs::EdjectModule(DWORD target_process_id, const char* module_file_absolute_path)
{
	BOOL	br;
	char*	file_name;
	HMODULE remote_module;

	file_name=(char*)module_file_absolute_path;

	{
		HANDLE			remote_snapshot;
		MODULEENTRY32	remote_module_info;

		remote_module=NULL;
		remote_snapshot=CreateToolhelp32Snapshot(TH32CS_SNAPMODULE,target_process_id);
		remote_module_info.dwSize=sizeof(remote_module_info);
		br=Module32First(remote_snapshot,&remote_module_info);
		return_if_equal(br,FALSE);
		do 
		{
			if(0 == lstrcmp(file_name,remote_module_info.szExePath) )
			{
				remote_module=remote_module_info.hModule;
				break;
			}

			remote_module_info.dwSize=sizeof(remote_module_info);
		}while(FALSE!=Module32Next(remote_snapshot,&remote_module_info));
		CloseHandle(remote_snapshot);
	}
	return_if_equal(remote_module,NULL);

	RunRemoteLibrary(target_process_id,false,remote_module);

}

void HookProcs::SetHook_AllModules(LPCTSTR module_which_provide_proc, 
						char* proc_name, FARPROC new_proc, FARPROC* old_proc)
{
	FARPROC				old_export_proc;
	const LPCTSTR		exporting_module_name=module_which_provide_proc;
	void*				exporting_module_base;


	{
		HMODULE dll=GetModuleHandle(exporting_module_name);
		exporting_module_base=(void*)((size_t)dll & 0xFFFF0000);
	}
	return_if_equal(exporting_module_base,NULL);

	{
		BOOL				br;
		HANDLE				snapshot;
		MODULEENTRY32		module_info;
		DWORD				process_id;

		process_id=GetCurrentProcessId();
		snapshot=CreateToolhelp32Snapshot(TH32CS_SNAPMODULE,process_id);
		module_info.dwSize=sizeof(module_info);
		br=Module32First(snapshot,&module_info);
		return_if_equal(br,FALSE);
		do 
		{
			FARPROC old_import_proc=NULL;
			AddHook_ImportTable_ModuleBase(module_info.modBaseAddr,exporting_module_name,proc_name,new_proc,&old_import_proc);
			module_info.dwSize=sizeof(module_info);
		}while(FALSE!=Module32Next(snapshot,&module_info));
		CloseHandle(snapshot);
	}



	AddHook_ExportTable_ModuleBase(exporting_module_base,proc_name,new_proc,&old_export_proc);
	if(old_proc!=NULL)
		*old_proc=old_export_proc;
}

