#pragma once

#include "HookProcs.h"
#include "IHookPool.h"
#include "Macros.h"


template <typename THookProc>
class THook: public IHook
{
public:
	THook(IHookPool* hook_pool,
				LPCTSTR module_which_provide_proc, char* proc_name, 
				THookProc* hook_proc, THookProc* original_proc);
	~THook();

	// IHook
	virtual void		SetHook();
	virtual void		ClearHook();
	virtual const char*	ProcName();
	virtual const char*	ModuleName();
	virtual bool		IsSeted();

	THookProc* const m_original;

private:
	LPCTSTR		m_module_name;
	char*		m_proc_name;
	THookProc*	m_hook_proc;
	IHookPool*	m_hook_pool;
	bool		m_seted;
};

//-------------------------------------------------------
template <typename THookProc>
THook<THookProc>::THook(IHookPool* hook_pool,
						LPCTSTR module_which_provide_proc, char* proc_name, 
						THookProc* hook_proc, THookProc* original_proc):
m_module_name(module_which_provide_proc),
m_proc_name(proc_name),
m_hook_proc(hook_proc),
m_seted(false),
m_hook_pool(hook_pool),
m_original(original_proc)// check of type identity
{
	if(m_hook_pool!=NULL)
		m_hook_pool->AddHook(this);
	else
		SetHook();
};

template <typename THookProc>
THook<THookProc>::~THook()
{
	if(m_hook_pool==NULL)
		ClearHook();
}

template <typename THookProc>
void THook<THookProc>::SetHook()
{
	return_if_equal(m_seted,true);
	HookProcs::SetHook_AllModules(m_module_name,m_proc_name,
									(FARPROC)(m_hook_proc),(FARPROC*)&m_original);
	m_seted=(m_original!=NULL);
}

template <typename THookProc>
void THook<THookProc>::ClearHook()
{
	HMODULE dll=GetModuleHandle(m_module_name);
	// dll=NULL it mean modul already unloaded
	if(dll!=NULL)
		HookProcs::SetHook_AllModules(m_module_name,m_proc_name,
									(FARPROC)(m_original),NULL);
	m_seted=false;
}

template <typename THookProc>
const char*	THook<THookProc>::ProcName()
{
	return m_module_name;
}

template <typename THookProc>
const char*	THook<THookProc>::ModuleName()
{
	return m_proc_name;
}

template <typename THookProc>
bool THook<THookProc>::IsSeted()
{
	return m_seted;
}
